searchData={"items":[{"type":"task","doc":"Generate Elixir code from `.proto` files.\n\nExample:\n`mix protox.generate --output-path=lib/message.ex --include-path=. message.proto`\n\nThe generated file will be usable in any project as long as protox is declared\nin the dependencies (the generated code still needs functions from the protox runtime).\n\nYou can use the `--namespace` option to prepend a namespace to all generated modules.\n\nIf you have large protobuf files, you can use the `--multiple-files` option to generate\none file per module (it will leverage parallel compilation).","title":"mix protox.generate","ref":"Mix.Tasks.Protox.Generate.html"},{"type":"module","doc":"Use this module to generate the Elixir structs corresponding to a set of protobuf definitions\nand to encode/decode instances of these structures.","title":"Protox","ref":"Protox.html"},{"type":"module","doc":"From a set of files:\n    defmodule Dummy do\n      use Protox,\n        files: [\n          \"./defs/foo.proto\",\n          \"./defs/bar.proto\",\n          \"./defs/baz/fiz.proto\",\n        ]\n    end\n\nFrom a string:\n    defmodule Dummy do\n      use Protox,\n        schema: \"\"\"\n        syntax = \"proto3\";\n        package fiz;\n\n        message Baz {\n        }\n\n        message Foo {\n          map  b = 2;\n        }\n        \"\"\"\n    end\n\nThe generated modules respect the package declaration. For instance, in the above example,\nboth the `Fiz.Baz` and `Fiz.Foo` modules will be generated.","title":"Elixit structs generation examples - Protox","ref":"Protox.html#module-elixit-structs-generation-examples"},{"type":"module","doc":"For the rest of this module documentation, we suppose the following protobuf messages are defined:\n    defmodule Dummy do\n      use Protox,\n        schema: \"\"\"\n          syntax = \"proto3\";\n          package fiz;\n\n          message Baz {\n          }\n\n          enum Enum {\n            FOO = 0;\n            BAR = 1;\n          }\n\n          message Foo {\n            Enum a = 1;\n            map  b = 2;\n          }\n        \"\"\",\n        namespace: Namespace\n\n      use Protox,\n        schema: \"\"\"\n        syntax = \"proto3\";\n\n        message Msg {\n          map  msg_k = 8;\n        }\n        \"\"\"\n\n      use Protox,\n        schema: \"\"\"\n        syntax = \"proto3\";\n\n        message Sub {\n          int32 a = 1;\n        }\n        \"\"\"\n    end\n\nSee each function documentation to see how they are used to encode and decode protobuf messages.","title":"Encoding/decoding - Protox","ref":"Protox.html#module-encoding-decoding"},{"type":"function","doc":"Decode a binary into a protobuf message.","title":"Protox.decode/2","ref":"Protox.html#decode/2"},{"type":"function","doc":"iex> binary = <<8, 42, 18, 7, 8, 1, 18, 3, 102, 111, 111>>\n    iex> {:ok, msg} = Protox.decode(binary, ProtoxExample)\n    iex> msg\n    %ProtoxExample{a: 42, b: %{1 => \"foo\"}}\n\n    iex> binary = <<66, 7, 8, 1, 18, 3, 102, 111, 66, 7, 8, 2, 18, 3, 98, 97, 114>>\n    iex> {:error, reason} = Protox.decode(binary, ProtoxExample)\n    iex> reason\n    %Protox.DecodingError{\n                  message: \"Could not decode data (invalid wire type 7)\",\n                  binary: <<7, 8, 2, 18, 3, 98, 97, 114>>\n                }","title":"Examples - Protox.decode/2","ref":"Protox.html#decode/2-examples"},{"type":"function","doc":"Throwing version of `decode/2`.","title":"Protox.decode!/2","ref":"Protox.html#decode!/2"},{"type":"function","doc":"Encode a protobuf message into IO data.","title":"Protox.encode/1","ref":"Protox.html#encode/1"},{"type":"function","doc":"iex> msg = %ProtoxExample{a: 3, b: %{1 => \"some string\"}}\n    iex> {:ok, iodata, _iodata_size} = Protox.encode(msg)\n    iex> IO.iodata_to_binary(iodata)\n    <<8, 3, 18, 15, 8, 1, 18, 11, 115, 111, 109, 101, 32, 115, 116, 114, 105, 110, 103>>\n\n    iex> msg = %ProtoxExample{a: \"should not be a string\"}\n    iex> {:error, reason} = Protox.encode(msg)\n    iex> reason\n    %Protox.EncodingError{field: :a, message: \"Could not encode field :a (invalid field value)\"}","title":"Examples - Protox.encode/1","ref":"Protox.html#encode/1-examples"},{"type":"function","doc":"Throwing version of `encode/1`.","title":"Protox.encode!/1","ref":"Protox.html#encode!/1"},{"type":"module","doc":"Default values of Protocol Buffers types.\n\nNote that generated structs contain a default/1 function to return the default\nvalue of a field using its name.","title":"Protox.Default","ref":"Protox.Default.html"},{"type":"function","doc":"Returns the default value of a Protocol Buffer type specified with an atom.","title":"Protox.Default.default/1","ref":"Protox.Default.html#default/1"},{"type":"function","doc":"iex> Protox.Default.default(:bool)\n    false\n\n    iex> Protox.Default.default(:string)\n    \"\"","title":"Examples - Protox.Default.default/1","ref":"Protox.Default.html#default/1-examples"},{"type":"module","doc":"The definition of a protobuf field (e.g. tag, type, etc.).","title":"Protox.Field","ref":"Protox.Field.html"},{"type":"type","doc":"","title":"Protox.Field.t/0","ref":"Protox.Field.html#t:t/0"},{"type":"module","doc":"Defines the kind of a field.\n\nIt can be one of the following:\n\n- `Protox.Scalar` - A [scalar](https://protobuf.dev/programming-guides/proto3/#scalar) value.\n- `:packed` - A [packed repeated](https://protobuf.dev/programming-guides/encoding/#packed) field.\n- `:unpacked` - An [unpacked repeated](https://protobuf.dev/programming-guides/encoding/#optional) field.\n- `:map` - A [map](https://protobuf.dev/programming-guides/encoding/#maps) field.\n- `Protox.OneOf` - A [oneof](https://protobuf.dev/programming-guides/encoding/#oneofs) field.","title":"Protox.Kind","ref":"Protox.Kind.html"},{"type":"type","doc":"","title":"Protox.Kind.t/0","ref":"Protox.Kind.html#t:t/0"},{"type":"module","doc":"This module provides a helper function to merge messages.","title":"Protox.MergeMessage","ref":"Protox.MergeMessage.html"},{"type":"function","doc":"Singular fields of `msg` will be overwritten, if specified in `from`, except for\nembedded messages which will be merged. Repeated fields will be concatenated.\n\nNote that \"specified\" has a different meaning in protobuf 2 and 3:\n- 2: if the singular field from `from` is nil, the value from `msg` is kept\n- 3: if the singular field from `from` is set to the default value, the value from `msg` is\nkept. This behaviour matches the C++ reference implementation behaviour.\n\n- `msg` and `from` must be of the same type; or\n- either `msg` or `from` is `nil`: the non-nil message is returned; or\n- both are `nil`: `nil` is returned\n\n# Example\n    iex> r1 = %Protobuf2Message{a: 0, b: :ONE}\n    iex> r2 = %Protobuf2Message{a: nil, b: :TWO}\n    iex> Protox.MergeMessage.merge(r1, r2)\n    %Protobuf2Message{a: 0, b: :TWO}\n    iex> Protox.MergeMessage.merge(r2, r1)\n    %Protobuf2Message{a: 0, b: :ONE}","title":"Protox.MergeMessage.merge/2","ref":"Protox.MergeMessage.html#merge/2"},{"type":"module","doc":"Represents the schema of a Protocol Buffers message once it has been processed by Protox.\n\nThis struct contains all the necessary information to describe a message in a Protocol Buffers\nschema, including its name, syntax version, fields, and optional file-level options.","title":"Protox.MessageSchema","ref":"Protox.MessageSchema.html"},{"type":"module","doc":"* `:name` - The atom representing the name of the message\n* `:syntax` - The Protocol Buffers syntax version (e.g., `:proto2` or `:proto3`)\n* `:fields` - A map of field names to their definitions (`Protox.Field.t()`)\n* `:file_options` - Optional file-level options, represented as a map if any.","title":"Fields - Protox.MessageSchema","ref":"Protox.MessageSchema.html#module-fields"},{"type":"type","doc":"","title":"Protox.MessageSchema.t/0","ref":"Protox.MessageSchema.html#t:t/0"},{"type":"module","doc":"Represents a [oneof field](https://protobuf.dev/programming-guides/proto3/#oneof) in protobuf.\n\nA oneof field represents a group of fields where only one of them can be set at a time.\nThis module provides a struct to store the parent field of this oneof group, that is, the\nfield that effectively contains the set value.","title":"Protox.OneOf","ref":"Protox.OneOf.html"},{"type":"module","doc":"* `:parent` - The name the parent field of this oneof group.","title":"Fields - Protox.OneOf","ref":"Protox.OneOf.html#module-fields"},{"type":"type","doc":"","title":"Protox.OneOf.t/0","ref":"Protox.OneOf.html#t:t/0"},{"type":"module","doc":"Represents a scalar field in a Protocol Buffer message.\n\nThis module defines a struct that holds information about a scalar field,\nparticularly its default value. Scalar fields are the basic data types\nsuch as integers, floats, booleans, strings.\n\nThe default value is used when a field is not present in the encoded message.","title":"Protox.Scalar","ref":"Protox.Scalar.html"},{"type":"type","doc":"All the possible types that can be used as a default value for a scalar.","title":"Protox.Scalar.scalar_default_value_type/0","ref":"Protox.Scalar.html#t:scalar_default_value_type/0"},{"type":"type","doc":"","title":"Protox.Scalar.t/0","ref":"Protox.Scalar.html#t:t/0"},{"type":"module","doc":"This module describes the types that define a protobuf message.\n\nSee https://developers.google.com/protocol-buffers/docs/encoding#structure.","title":"Protox.Types","ref":"Protox.Types.html"},{"type":"type","doc":"This type gives more information on the field presence (if applicable).","title":"Protox.Types.label/0","ref":"Protox.Types.html#t:label/0"},{"type":"type","doc":"All types that can be used as a key in a map field.","title":"Protox.Types.map_key/0","ref":"Protox.Types.html#t:map_key/0"},{"type":"type","doc":"The wire type of a field: it tells how a field is encoded (32 or 64 bits scalar, repeated or\nvariable-length integer).","title":"Protox.Types.tag/0","ref":"Protox.Types.html#t:tag/0"},{"type":"type","doc":"All the types that can be stored in a protobuf message.","title":"Protox.Types.type/0","ref":"Protox.Types.html#t:type/0"},{"type":"type","doc":"32 bits scalar.","title":"Protox.Types.wire_32bits/0","ref":"Protox.Types.html#t:wire_32bits/0"},{"type":"type","doc":"64 bits scalar.","title":"Protox.Types.wire_64bits/0","ref":"Protox.Types.html#t:wire_64bits/0"},{"type":"type","doc":"Repeated field.","title":"Protox.Types.wire_delimited/0","ref":"Protox.Types.html#t:wire_delimited/0"},{"type":"type","doc":"Variable length integer.","title":"Protox.Types.wire_varint/0","ref":"Protox.Types.html#t:wire_varint/0"},{"type":"exception","doc":"This error is thrown when a data could not be decoded.","title":"Protox.DecodingError","ref":"Protox.DecodingError.html"},{"type":"exception","doc":"This error is thrown when a message could not be encoded.","title":"Protox.EncodingError","ref":"Protox.EncodingError.html"},{"type":"exception","doc":"This error is thrown when decoding data with a field which tag is 0.","title":"Protox.IllegalTagError","ref":"Protox.IllegalTagError.html"},{"type":"exception","doc":"This error is thrown when a field is constructed with an invalid atribute.","title":"Protox.InvalidFieldAttributeError","ref":"Protox.InvalidFieldAttributeError.html"},{"type":"exception","doc":"This error is thrown when encoding or decoding a Protobuf 2 message\nwith unset required fields (that is, that have the value `nil`).","title":"Protox.RequiredFieldsError","ref":"Protox.RequiredFieldsError.html"},{"type":"extras","doc":"# Changelog\n\nAll notable changes to this project will be documented in this file.\n\nThe format is based on [Keep a Changelog](https://keepachangelog.com/en/1.1.0/),\nand this project adheres to [Semantic Versioning](https://semver.org/spec/v2.0.0.html).","title":"Changelog","ref":"changelog.html"},{"type":"extras","doc":"","title":"2.0.1 - Changelog","ref":"changelog.html#2-0-1"},{"type":"extras","doc":"- Correct declaration of benchee dependency which is required at runtime","title":"Fixed - Changelog","ref":"changelog.html#fixed"},{"type":"extras","doc":"","title":"2.0.0 - Changelog","ref":"changelog.html#2-0-0"},{"type":"extras","doc":"- `schema/0` to access the underlying definition of a message.","title":"Added - Changelog","ref":"changelog.html#added"},{"type":"extras","doc":"- Minimal supported Elixir version is now 1.15.\n- BREAKING CHANGE: encoding functions now return the size alongside iodata.\nPreviously, one had to use `:binary.list_to_bin/1` to flatten the iodata and then use `byte_size`, defeating the purpose of having an iodata. Furthermore, it provides a significant performance boost.\n- BREAKING CHANGE: `Protox.decode!` and `Protox.decode`  no longer raise or return an error if a message don't have all required fields set. We should let the user decide whether it's a problem.\n- BREAKING CHANGE: Rename `InvalidFieldAttribute` to `InvalidFieldAttributeError` for consistency.","title":"Changed - Changelog","ref":"changelog.html#changed"},{"type":"extras","doc":"- BREAKING CHANGE: Drop support of JSON encoding and decoding.\n  The implementation was half-baked, it's preferable to have the library focused. Even [Google doesn't recommend](https://protobuf.dev/programming-guides/json/) using JSON: _\"In short, there are many good reasons why Google prefers to use the standard wire format for virtually everything rather than ProtoJSON format\"_.\n- BREAKING CHANGE: Remove `:path` option in favor of the already existing `:paths` option, which covers the same functionality.\n- BREAKING CHANGE: Remove `:keep_unknown_fields` option (the corresponding `--keep-unknown-fields` option for the mix task `protox.generate` is also removed). They are now always kept.\nIt added complexity to the generation logic while not providing any value as one can simply ignore those fields and as it's mandatory to parse them correctly.\n- BREAKING CHANGE: Remove generated `defs/0` (functionality remains available through `schema/0`).\n- BREAKING CHANGE: Remove generated `field_def/1` (functionality remains available through `schema/0`).\n- BREAKING CHANGE: Remove generated `file_options/0` (functionality remains available through `schema/0`).\n- BREAKING CHANGE: Remove generated `required_fields/0` (functionality remains available through `schema/0`).\n- BREAKING CHANGE: Remove generated `syntax/0` (functionality remains available through `schema/0`).\n- BREAKING CHANGE: Remove generated `encode/1` from strings for enums.\n- BREAKING CHANGE: Remove `Protox.Encode.encode/1` and `Protox.Encode.encode!/1` (functionality remains available in generated modules and in `Protox`).","title":"Removed - Changelog","ref":"changelog.html#removed"},{"type":"extras","doc":"","title":"1.7.8 - Changelog","ref":"changelog.html#1-7-8"},{"type":"extras","doc":"- Fix warnings of unknown JSON modules when a JSON library is not installed","title":"Fixed - Changelog","ref":"changelog.html#fixed"},{"type":"extras","doc":"","title":"1.7.7 - Changelog","ref":"changelog.html#1-7-7"},{"type":"extras","doc":"- Fix launch by removing :propcheck from extra applications","title":"Fixed - Changelog","ref":"changelog.html#fixed"},{"type":"extras","doc":"","title":"1.7.6 - Changelog","ref":"changelog.html#1-7-6"},{"type":"extras","doc":"- Fix typespecs for JSON decoding (thanks to https://github.com/squirmy)","title":"Fixed - Changelog","ref":"changelog.html#fixed"},{"type":"extras","doc":"","title":"1.7.5 - Changelog","ref":"changelog.html#1-7-5"},{"type":"extras","doc":"- Use origin repository for propchek rather than a fork","title":"Changed - Changelog","ref":"changelog.html#changed"},{"type":"extras","doc":"","title":"1.7.4 - Changelog","ref":"changelog.html#1-7-4"},{"type":"extras","doc":"- Fix handling of enum in snake case","title":"Fixed - Changelog","ref":"changelog.html#fixed"},{"type":"extras","doc":"","title":"1.7.3 - Changelog","ref":"changelog.html#1-7-3"},{"type":"extras","doc":"- Raise DecodingError and EncodingError for invalid strings (thanks to https://github.com/g-andrade)","title":"Added - Changelog","ref":"changelog.html#added"},{"type":"extras","doc":"","title":"1.7.2 - Changelog","ref":"changelog.html#1-7-2"},{"type":"extras","doc":"- Fix typespec of enum encode function (thanks to https://github.com/wingyplus)","title":"Fixed - Changelog","ref":"changelog.html#fixed"},{"type":"extras","doc":"","title":"1.7.1 - Changelog","ref":"changelog.html#1-7-1"},{"type":"extras","doc":"- Fix decoding and encoding of proto3 optional fields","title":"Fixed - Changelog","ref":"changelog.html#fixed"},{"type":"extras","doc":"","title":"1.7.0 - Changelog","ref":"changelog.html#1-7-0"},{"type":"extras","doc":"- Support FileOptions (which can be access with `Msg.file_options/0`)","title":"Added - Changelog","ref":"changelog.html#added"},{"type":"extras","doc":"","title":"1.6.10 - Changelog","ref":"changelog.html#1-6-10"},{"type":"extras","doc":"- Format of generated files","title":"Changed - Changelog","ref":"changelog.html#changed"},{"type":"extras","doc":"","title":"1.6.9 - Changelog","ref":"changelog.html#1-6-9"},{"type":"extras","doc":"- Fix deprecation warnings in Elixir >= 1.14 about Bitwise (thanks to https://github.com/moogle19)","title":"Fixed - Changelog","ref":"changelog.html#fixed"},{"type":"extras","doc":"","title":"1.6.8 - Changelog","ref":"changelog.html#1-6-8"},{"type":"extras","doc":"- Raise clearer error message if protoc is missing (thanks to https://github.com/josevalim)","title":"Added - Changelog","ref":"changelog.html#added"},{"type":"extras","doc":"","title":"1.6.7 - Changelog","ref":"changelog.html#1-6-7"},{"type":"extras","doc":"- New release to publish docs using the improved ex_doc 0.27","title":"Changed - Changelog","ref":"changelog.html#changed"},{"type":"extras","doc":"","title":"1.6.6 - Changelog","ref":"changelog.html#1-6-6"},{"type":"extras","doc":"- Fix JSON conformance tests related to fractional part in Timestamp","title":"Fixed - Changelog","ref":"changelog.html#fixed"},{"type":"extras","doc":"- Option to not generate deprecated functions `defs/0` and `defs_by_name/0`","title":"Added - Changelog","ref":"changelog.html#added"},{"type":"extras","doc":"","title":"1.6.5 - Changelog","ref":"changelog.html#1-6-5"},{"type":"extras","doc":"- Elixir 1.9 is now the minimal supported version\n- Relax constraint on Decimal version (thanks to https://github.com/ananthakumaran)","title":"Changed - Changelog","ref":"changelog.html#changed"},{"type":"extras","doc":"","title":"1.6.4 - Changelog","ref":"changelog.html#1-6-4"},{"type":"extras","doc":"- Renamed module Protox.Message into Protox.MergeMessage to reflect its real role","title":"Changed - Changelog","ref":"changelog.html#changed"},{"type":"extras","doc":"- Fix inconsistent behavior when encoding to JSON an enum with an unknown atom field (WARNING: Requires to regenerate code from .proto definitions)","title":"Fixed - Changelog","ref":"changelog.html#fixed"},{"type":"extras","doc":"","title":"1.6.3 - Changelog","ref":"changelog.html#1-6-3"},{"type":"extras","doc":"- Fix typespec of message's json_decode! function","title":"Fixed - Changelog","ref":"changelog.html#fixed"},{"type":"extras","doc":"","title":"1.6.2 - Changelog","ref":"changelog.html#1-6-2"},{"type":"extras","doc":"- Fix possible double compilation of Empty well-known type\n- Fix dependency on protoc for generated code","title":"Fixed - Changelog","ref":"changelog.html#fixed"},{"type":"extras","doc":"","title":"1.6.1 - Changelog","ref":"changelog.html#1-6-1"},{"type":"extras","doc":"- Fix compilation when protoc does not include well-known types","title":"Fixed - Changelog","ref":"changelog.html#fixed"},{"type":"extras","doc":"","title":"1.6.0 - Changelog","ref":"changelog.html#1-6-0"},{"type":"extras","doc":"- Add support of JSON protobuf encoding and decoding (https://developers.google.com/protocol-buffers/docs/proto3##json), with support of well-known types (except for Any)","title":"Added - Changelog","ref":"changelog.html#added"},{"type":"extras","doc":"- More accurate error reporting\n- Internal refactoring to hopefully make things more explicit (based on a work initiated by https://github.com/sneako)","title":"Changed - Changelog","ref":"changelog.html#changed"},{"type":"extras","doc":"- Fix decoding of fixed32 and fixed64 values (detected using JSON conformance tests)","title":"Fixed - Changelog","ref":"changelog.html#fixed"},{"type":"extras","doc":"- `Protox.Encode.encode/1` and `Protox.Encode.encode!/1`; use `Protox.encode/1` and `Protox.encode!/1` instead\n-  Generated functions`defs/0` and `defs_by_name/0`","title":"Deprecated - Changelog","ref":"changelog.html#deprecated"},{"type":"extras","doc":"","title":"1.5.1 - Changelog","ref":"changelog.html#1-5-1"},{"type":"extras","doc":"Fix handling of multiple import paths (thanks to https://github.com/zolakeith)","title":"Fixed - Changelog","ref":"changelog.html#fixed"},{"type":"extras","doc":"","title":"1.5.0 - Changelog","ref":"changelog.html#1-5-0"},{"type":"extras","doc":"- Allow multiple import paths (thanks to https://github.com/cheng81)","title":"Added - Changelog","ref":"changelog.html#added"},{"type":"extras","doc":"","title":"1.4.0 - Changelog","ref":"changelog.html#1-4-0"},{"type":"extras","doc":"- Add support of proto3 optional fields (thanks to https://github.com/sneako)","title":"Added - Changelog","ref":"changelog.html#added"},{"type":"extras","doc":"","title":"1.3.2 - Changelog","ref":"changelog.html#1-3-2"},{"type":"extras","doc":"- Bump version to build doc using ex_doc 0.24","title":"Changed - Changelog","ref":"changelog.html#changed"},{"type":"extras","doc":"","title":"1.3.1 - Changelog","ref":"changelog.html#1-3-1"},{"type":"extras","doc":"- Fix table of types mapping in documentation","title":"Fixed - Changelog","ref":"changelog.html#fixed"},{"type":"extras","doc":"","title":"1.3.0 - Changelog","ref":"changelog.html#1-3-0"},{"type":"extras","doc":"- Allow namespaces through protox.generate (thanks to https://github.com/sdrew)","title":"Added - Changelog","ref":"changelog.html#added"},{"type":"extras","doc":"- Expand output path when generating files\n- More thorough testing of code generation","title":"Changed - Changelog","ref":"changelog.html#changed"},{"type":"extras","doc":"","title":"1.2.4 - Changelog","ref":"changelog.html#1-2-4"},{"type":"extras","doc":"- Format generated code","title":"Changed - Changelog","ref":"changelog.html#changed"},{"type":"extras","doc":"- Fix warning when compiling generated code (thanks to https://github.com/xinz)\n- Fix warning about unused variable in generated code when encoding an empty protobuf message","title":"Fixed - Changelog","ref":"changelog.html#fixed"},{"type":"extras","doc":"","title":"1.2.3 - Changelog","ref":"changelog.html#1-2-3"},{"type":"extras","doc":"- Update documentation to better explain the package directive usage","title":"Changed - Changelog","ref":"changelog.html#changed"},{"type":"extras","doc":"","title":"1.2.2 - Changelog","ref":"changelog.html#1-2-2"},{"type":"extras","doc":"- Enable listing of task protox.generate via mix help.","title":"Added - Changelog","ref":"changelog.html#added"},{"type":"extras","doc":"","title":"1.2.1 - Changelog","ref":"changelog.html#1-2-1"},{"type":"extras","doc":"- `--keep-unknown-fields` option to configure support of unknown fields when generating files","title":"Added - Changelog","ref":"changelog.html#added"},{"type":"extras","doc":"","title":"1.2.0 - Changelog","ref":"changelog.html#1-2-0"},{"type":"extras","doc":"- Add `:keep_unknown_fields` option to configure support of unknown fields","title":"Added - Changelog","ref":"changelog.html#added"},{"type":"extras","doc":"","title":"1.1.1 - Changelog","ref":"changelog.html#1-1-1"},{"type":"extras","doc":"- Fix documentation links","title":"Fixed - Changelog","ref":"changelog.html#fixed"},{"type":"extras","doc":"","title":"1.1.0 - Changelog","ref":"changelog.html#1-1-0"},{"type":"extras","doc":"- It's now possible to generate one file per protobuf message to speed up compilation (thanks to https://github.com/qgau)","title":"Added - Changelog","ref":"changelog.html#added"},{"type":"extras","doc":"","title":"1.0.0 - Changelog","ref":"changelog.html#1-0-0"},{"type":"extras","doc":"- Use Protox exceptions as errors codes","title":"Changed - Changelog","ref":"changelog.html#changed"},{"type":"extras","doc":"","title":"0.25.0 - Changelog","ref":"changelog.html#0-25-0"},{"type":"extras","doc":"- Add mix task to generate files","title":"Added - Changelog","ref":"changelog.html#added"},{"type":"extras","doc":"- Bump to Elixir 1.7 as minimal supported version","title":"Changed - Changelog","ref":"changelog.html#changed"},{"type":"extras","doc":"","title":"0.24.0 - Changelog","ref":"changelog.html#0-24-0"},{"type":"extras","doc":"- Usage of `@external_resource` is no longer necessary","title":"Changed - Changelog","ref":"changelog.html#changed"},{"type":"extras","doc":"","title":"0.23.1 - Changelog","ref":"changelog.html#0-23-1"},{"type":"extras","doc":"- Fix parse of `[packed=false]` option (the serialization was correct, but not in compliance with Protobuf conformance checker recommandations)","title":"Fixed - Changelog","ref":"changelog.html#fixed"},{"type":"extras","doc":"","title":"0.23.0 - Changelog","ref":"changelog.html#0-23-0"},{"type":"extras","doc":"- BREAKING CHANGE: `encode/1` returns a tuple, use `encode!/1` to get the old behavior of `encode/1`\n- +40% speedup & -30% memory consumption when decoding thanks to macros\n- Raise RequiredFieldsError when encoding or decoding a Protobuf 2 message with unset required fields (that is, that have the value `nil`)\n- Raise IllegalTagError when decoding a message with a tag set to `0`","title":"Changed - Changelog","ref":"changelog.html#changed"},{"type":"extras","doc":"- Fix missing encoding of unknown fields when a message hadn't any field","title":"Fixed - Changelog","ref":"changelog.html#fixed"},{"type":"extras","doc":"","title":"0.22.0 - Changelog","ref":"changelog.html#0-22-0"},{"type":"extras","doc":"- Constant time encoding of oneof fields","title":"Changed - Changelog","ref":"changelog.html#changed"},{"type":"extras","doc":"","title":"0.21.0 - Changelog","ref":"changelog.html#0-21-0"},{"type":"extras","doc":"- Move back to ahamez/protox\n- Bump to Elixir 1.6 as minimal supported version","title":"Changed - Changelog","ref":"changelog.html#changed"},{"type":"extras","doc":"- Add benchmarks\n- Add conformance tests to CI\n- Add dialyzer to CI","title":"Added - Changelog","ref":"changelog.html#added"},{"type":"extras","doc":"","title":"0.20.0 - Changelog","ref":"changelog.html#0-20-0"},{"type":"extras","doc":"- Pass all tests of protobuf 3.12 conformance suite tests\n- Always serialize required fields (proto2)","title":"Fixed - Changelog","ref":"changelog.html#fixed"},{"type":"extras","doc":"- `defs_by_name/0` in generated modules for messages\n- `syntax/0` in generated modules for messages\n- `Protox.MergeMessage.merge/2` to merge two messages of the same type","title":"Added - Changelog","ref":"changelog.html#added"},{"type":"extras","doc":"- BREAKING CHANGE: (proto2) use nil for unset fields\n- BREAKING CHANGE: rename generated `get_required_fields/0` into `required_fields/0`\n- BREAKING CHANGE: rename generated `get_unknown_fields/0` into `unknown_fields/0`\n- BREAKING CHANGE: rename generated `get_unknown_fields_name/0` into `unknown_fields_name/0`","title":"Changed - Changelog","ref":"changelog.html#changed"},{"type":"extras","doc":"","title":"0.19.1 - Changelog","ref":"changelog.html#0-19-1"},{"type":"extras","doc":"- Fix warning about duplicate keys (thanks to https://github.com/ananthakumaran)","title":"Fixed - Changelog","ref":"changelog.html#fixed"},{"type":"extras","doc":"","title":"0.19.0 - Changelog","ref":"changelog.html#0-19-0"},{"type":"extras","doc":"- CamelCase for all generated modules (fixes https://github.com/ahamez/protox/issues/3)","title":"Changed - Changelog","ref":"changelog.html#changed"},{"type":"extras","doc":"","title":"0.18.0 - Changelog","ref":"changelog.html#0-18-0"},{"type":"extras","doc":"- Allow ability to construct file names at compile time (thanks to https://github.com/ananthakumaran)","title":"Added - Changelog","ref":"changelog.html#added"},{"type":"extras","doc":"","title":"0.17.0 - Changelog","ref":"changelog.html#0-17-0"},{"type":"extras","doc":"- `:path` option to specify import path (thanks to https://github.com/mathsaey)","title":"Added - Changelog","ref":"changelog.html#added"},{"type":"extras","doc":"","title":"0.16.2 - Changelog","ref":"changelog.html#0-16-2"},{"type":"extras","doc":"- Fix generation of typespecs for when there are more than one required field","title":"Fixed - Changelog","ref":"changelog.html#fixed"},{"type":"extras","doc":"","title":"0.16.1 - Changelog","ref":"changelog.html#0-16-1"},{"type":"extras","doc":"- Change base name for unknown fields from `__unknown_fields__` to `__uf__`","title":"Changed - Changelog","ref":"changelog.html#changed"},{"type":"extras","doc":"","title":"0.16.0 - Changelog","ref":"changelog.html#0-16-0"},{"type":"extras","doc":"- Fix handling of +/-infinity and NaN when encoding/decoding floats\n\n#### Changed\n- Move `RandomInit` to tests","title":"Fixed - Changelog","ref":"changelog.html#fixed"},{"type":"extras","doc":"","title":"0.15.2 - Changelog","ref":"changelog.html#0-15-2"},{"type":"extras","doc":"- Fix typespecs for enum constants accessors","title":"Fixed - Changelog","ref":"changelog.html#fixed"},{"type":"extras","doc":"","title":"0.15.1 - Changelog","ref":"changelog.html#0-15-1"},{"type":"extras","doc":"- Fix typespecs for unknown and required fields accessors","title":"Fixed - Changelog","ref":"changelog.html#fixed"},{"type":"extras","doc":"#### Changed\n- Use `0.0` as default value for floats and doubles","title":"0.15.0 - Changelog","ref":"changelog.html#0-15-0"},{"type":"extras","doc":"","title":"0.14.0 - Changelog","ref":"changelog.html#0-14-0"},{"type":"extras","doc":"- Development now takes place at https://github.com/EasyMile/protox\n- Move conformance test escript to https://github.com/EasyMile/protox-conformance","title":"Changed - Changelog","ref":"changelog.html#changed"},{"type":"extras","doc":"- Benchmarks escripts","title":"Removed - Changelog","ref":"changelog.html#removed"},{"type":"extras","doc":"","title":"0.13.0 - Changelog","ref":"changelog.html#0-13-0"},{"type":"extras","doc":"- Typespecs for generated encoder\n- Bring `varint` library into `protox`","title":"Added - Changelog","ref":"changelog.html#added"},{"type":"extras","doc":"- Fix decoding of booleans encoded with a varint which is not `0` or `1`","title":"Fixed - Changelog","ref":"changelog.html#fixed"},{"type":"extras","doc":"","title":"0.12.1 - Changelog","ref":"changelog.html#0-12-1"},{"type":"extras","doc":"- Fix handling of unset members in map entries","title":"Fixed - Changelog","ref":"changelog.html#fixed"},{"type":"extras","doc":"#### Changed\n- ~2x speed improvement when encoding","title":"0.12.0 - Changelog","ref":"changelog.html#0-12-0"},{"type":"extras","doc":"","title":"0.11.1 - Changelog","ref":"changelog.html#0-11-1"},{"type":"extras","doc":"- It's now possible to clear unknown fields","title":"Added - Changelog","ref":"changelog.html#added"},{"type":"extras","doc":"","title":"0.11.0 - Changelog","ref":"changelog.html#0-11-0"},{"type":"extras","doc":"- Encode unknown fields","title":"Added - Changelog","ref":"changelog.html#added"},{"type":"extras","doc":"","title":"0.10.0 - Changelog","ref":"changelog.html#0-10-0"},{"type":"extras","doc":"- Update deps (dialyxir, excoveralls, hackney)","title":"Changed - Changelog","ref":"changelog.html#changed"},{"type":"extras","doc":"","title":"0.9.0 - Changelog","ref":"changelog.html#0-9-0"},{"type":"extras","doc":"- Keep unknown fields when decoding","title":"Added - Changelog","ref":"changelog.html#added"},{"type":"extras","doc":"","title":"0.8.0 - Changelog","ref":"changelog.html#0-8-0"},{"type":"extras","doc":"- Raise an error when decoding and when required fields are missing","title":"Changed - Changelog","ref":"changelog.html#changed"},{"type":"extras","doc":"","title":"0.7.1 - Changelog","ref":"changelog.html#0-7-1"},{"type":"extras","doc":"- Fix encoding of varint to match C++ version\n- Fix encoding of enums to match C++ version","title":"Fixed - Changelog","ref":"changelog.html#fixed"},{"type":"extras","doc":"","title":"0.7.0 - Changelog","ref":"changelog.html#0-7-0"},{"type":"extras","doc":"- Read definitions from files or binaries\n- Parse definitions with protoc\n- Generate Elixir structs from parsed definition\n- Can prepend namespaces\n- Encode/decode protobuf messages","title":"Added - Changelog","ref":"changelog.html#added"},{"type":"extras","doc":"# How to contribute to protox\nFirst, thank you for your interest in contributing to protox!\n\nTo ensure a smooth experience while contributing, here are a few handy guidelines.","title":"How to contribute to protox","ref":"contributing.html"},{"type":"extras","doc":"- Erlang/OTP 26 or later\n- Elixir 1.15 or later\n- (optional) [lefthook](https://evilmartians.github.io/lefthook/installation) for git hooks","title":"Development Prerequisites - How to contribute to protox","ref":"contributing.html#development-prerequisites"},{"type":"extras","doc":"We use a few tools to keep the code clean and consistent:\n- [`mix deps.unlock --check-unused`](https://hexdocs.pm/mix/Mix.Tasks.Deps.Unlock.html) to check for unused dependencies\n- [`mix format --check-formatted`](https://hexdocs.pm/mix/Mix.Tasks.Format.html) to check for code formatting issues\n- [`mix credo`](https://hexdocs.pm/credo/Mix.Tasks.Credo.html) for code style and consistency checks\n- [`mix dialyzer`](https://hexdocs.pm/dialyxir/Mix.Tasks.Dialyzer.html) for type checking\n- [`mix muzak`](https://hexdocs.pm/muzak_pro/muzak-pro.html) for mutation testing\n- `mix test --include conformance` for testing\n\n> [!NOTE]\n>\n> These tasks are always run in the CI pipeline.\n\n> [!NOTE]\n>\n> `lefthook` can be used to run these tasks automatically on each commit or push (except for the `muzak` task which takes a long time to run).\n\n> [!NOTE]\n>\n> `mix test --include conformance` automatically downloads and compiles the conformance test suite and runs it against the current version of protox.","title":"Development Guidelines - How to contribute to protox","ref":"contributing.html#development-guidelines"},{"type":"extras","doc":"Correctness is the main goal of Protox, here's how you can contribute to it:\n- add tests for any new features;\n- when fixing a bug, add tests that reproduce the bug;\n- ensure all tests pass with `mix test --include conformance`;\n- try to maintain or improve test coverage (check with `mix test --cover`).","title":"Testing - How to contribute to protox","ref":"contributing.html#testing"},{"type":"extras","doc":"Documentation is as important as correctness, here's a quick reminder of the things to keep in mind:\n- document public functions;\n- update module documentation if needed;\n- if possible, include examples as [doctests](https://hexdocs.pm/ex_unit/ExUnit.DocTest.html);\n- update the main [README.md](./README.md) if needed.","title":"Documentation - How to contribute to protox","ref":"contributing.html#documentation"},{"type":"extras","doc":"By contributing to Protox, you agree that your contributions will be licensed under MIT License.","title":"License - How to contribute to protox","ref":"contributing.html#license"},{"type":"extras","doc":"If you have questions or need help, you can:\n- Send me a direct message on [Elixir Forum](https://elixirforum.com/u/ahamez).\n- Send me an email at alexandre.hamez at gmail.com.\n- Open an issue.\n- Start a discussion at https://github.com/ahamez/protox/discussions.","title":"Getting Help - How to contribute to protox","ref":"contributing.html#getting-help"},{"type":"extras","doc":"# Protox\n\n[![Elixir CI](https://github.com/ahamez/protox/actions/workflows/elixir.yml/badge.svg)](https://github.com/ahamez/protox/actions/workflows/elixir.yml) [![Coverage Status](https://coveralls.io/repos/github/ahamez/protox/badge.svg?branch=master)](https://coveralls.io/github/ahamez/protox?branch=master) [![Hex.pm Version](http://img.shields.io/hexpm/v/protox.svg)](https://hex.pm/packages/protox) [![Hex Docs](https://img.shields.io/badge/hex-docs-brightgreen.svg)](https://hexdocs.pm/protox/) [![License](https://img.shields.io/hexpm/l/protox.svg)](https://github.com/ahamez/protox/blob/master/LICENSE)\n\nProtox is an Elixir library for working with [Google's Protocol Buffers](https://developers.google.com/protocol-buffers), versions 2 and 3, supporting binary encoding and decoding.\n\nThe primary objective of Protox is **reliability**: it uses [property testing](https://github.com/alfert/propcheck), [mutation testing](https://github.com/devonestes/muzak) and has a [near 100% code coverage](https://coveralls.io/github/ahamez/protox?branch=master). Protox [passes all the tests](#conformance) of the conformance checker provided by Google.\n\n\n> [!NOTE]\n> If you're using version 1, please see how to migrate to version 2 [here](documentation/v1_to_v2_migration.md).","title":"Protox","ref":"readme.html"},{"type":"extras","doc":"Given the following protobuf definition:\n```proto\nmessage Msg{\n  int32 a = 1;\n  map  b = 2;\n}\n```\n\nProtox will create a regular Elixir `Msg` struct:\n```elixir\niex> msg = %Msg{a: 42, b: %{1 => \"a map entry\"}}\niex> {:ok, iodata, iodata_size} = Msg.encode(msg)\n\niex> binary = # read binary from a socket, a file, etc.\niex> {:ok, msg} = Msg.decode(binary)\n```","title":"Example - Protox","ref":"readme.html#example"},{"type":"extras","doc":"You can use Protox in two ways:\n\n1. pass the protobuf schema ([as an inlined schema](#usage-with-an-inlined-schema) or as a [list of files](#usage-with-files)) to the `Protox` macro;\n2. [generate](#files-generation) Elixir source code files with the mix task `protox.generate`.","title":"Usage - Protox","ref":"readme.html#usage"},{"type":"extras","doc":"- [Prerequisites](#prerequisites)\n- [Installation](#installation)\n- [Usage with an inlined schema](#usage-with-an-inlined-schema)\n- [Usage with files](#usage-with-files)\n- [Encode](#encode)\n- [Decode](#decode)\n- [Packages and namespaces](#packages-and-namespaces)\n- [Specify include path](#specify-include-path)\n- [Files generation](#files-generation)\n- [Unknown fields](#unknown-fields)\n- [Unsupported features](#unsupported-features)\n- [Implementation choices](#implementation-choices)\n- [Generated code reference and types mapping](#generated-code-reference-and-types-mapping)\n- [Conformance](#conformance)\n- [Benchmark](#benchmark)\n- [Contributing](#contributing)","title":"Table of contents - Protox","ref":"readme.html#table-of-contents"},{"type":"extras","doc":"- Elixir >= 1.15 and OTP >= 26\n- [protoc](https://github.com/protocolbuffers/protobuf/releases) >= 3.0 *This dependency is only required at compile-time*. It must be available in `$PATH`.","title":"Prerequisites - Protox","ref":"readme.html#prerequisites"},{"type":"extras","doc":"Add `:protox` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [{:protox, \"~> 2.0\"}]\nend\n```","title":"Installation - Protox","ref":"readme.html#installation"},{"type":"extras","doc":"The following example generates two modules, `Baz` and `Foo`:\n\n```elixir\ndefmodule MyModule do\n  use Protox, schema: \"\"\"\n  syntax = \"proto3\";\n\n  message Baz {\n  }\n\n  message Foo {\n    int32 a = 1;\n    map  b = 2;\n  }\n  \"\"\"\nend\n```\n\n> [!NOTE]\n> The module in which the `Protox` macro is called is ignored and does not appear in the names of the generated modules. To include the enclosing module’s name, use the `namespace` option, see [here](#prepend-namespaces).","title":"Usage with an inlined schema - Protox","ref":"readme.html#usage-with-an-inlined-schema"},{"type":"extras","doc":"Use the `:files` option to pass a list of files:\n\n```elixir\ndefmodule MyModule do\n  use Protox, files: [\n    \"./defs/foo.proto\",\n    \"./defs/bar.proto\",\n    \"./defs/baz/fiz.proto\"\n  ]\nend\n```","title":"Usage with files - Protox","ref":"readme.html#usage-with-files"},{"type":"extras","doc":"Here's how to encode a message to binary protobuf:\n\n```elixir\nmsg = %Foo{a: 3, b: %{1 => %Baz{}}}\n{:ok, iodata, iodata_size} = Protox.encode(msg)\n# or using the bang version\n{iodata, iodata_size} = Protox.encode!(msg)\n```\n\nIt's also possible to call `encode/1` and `encode!/1` directly on the generated structures:\n\n```elixir\n{:ok, iodata, iodata_size} = Foo.encode(msg)\n{iodata, iodata_size} = Foo.encode!(msg)\n```\n\n> [!NOTE]\n> `encode/1` and `encode!/1` return an [IO data](https://hexdocs.pm/elixir/IO.html#module-use-cases-for-io-data) for efficiency reasons. Such IO data can be used directly with files or sockets write operations:\n> ```elixir\n> iex> {iodata, _iodata_size} = Protox.encode!(%Foo{a: 3, b: %{1 => %Baz{}}})\n> {[\"\\b\", <<3>>, <<18, 4, 8>>, <<1>>, <<18>>, [<<0>>, []]], 8}\n> iex> {:ok, file} = File.open(\"msg.bin\", [:write])\n> {:ok, #PID<0.1023.0>}\n> iex> IO.binwrite(file, iodata)\n> :ok\n> ```\n>\n> Use [`:binary.list_to_bin/1`](https://erlang.org/doc/man/binary.html#list_to_bin-1) or [`IO.iodata_to_binary`](https://hexdocs.pm/elixir/IO.html#iodata_to_binary/1) if you need to get a binary from an IO data.","title":"Encode - Protox","ref":"readme.html#encode"},{"type":"extras","doc":"Here's how to decode a message from binary protobuf:\n\n```elixir\n{:ok, msg} = Protox.decode(<<8, 3, 18, 4, 8, 1, 18, 0>>, Foo)\n# or using the bang version\nmsg = Protox.decode!(<<8, 3, 18, 4, 8, 1, 18, 0>>, Foo)\n```\n\nIt's also possible to call `decode/1` and `decode!/1` directly on the generated structures:\n\n```elixir\n{:ok, msg} = Foo.decode(<<8, 3, 18, 4, 8, 1, 18, 0>>)\nmsg = Foo.decode!(<<8, 3, 18, 4, 8, 1, 18, 0>>)\n```","title":"Decode - Protox","ref":"readme.html#decode"},{"type":"extras","doc":"","title":"Packages and namespaces - Protox","ref":"readme.html#packages-and-namespaces"},{"type":"extras","doc":"Protox honors the [`package`](https://protobuf.dev/programming-guides/proto3/#packages) directive:\n\n```proto\npackage abc.def;\nmessage Baz {}\n```\n\nThe example above will be translated to `Abc.Def.Baz` (note the [camelization](#implementation-choices) of package `abc.def` to `Abc.Def`).","title":"Packages - Protox","ref":"readme.html#packages"},{"type":"extras","doc":"In addition, Protox provides the possibility to prepend a namespace with the `:namespace` option:\n\n```elixir\ndefmodule Bar do\n  use Protox, schema: \"\"\"\n    syntax = \"proto3\";\n\n    package abc;\n\n    message Msg {\n        int32 a = 1;\n      }\n    \"\"\",\n    namespace: __MODULE__\nend\n```\n\nIn this example, the module `Bar.Abc.Msg` is generated:\n\n```elixir\nmsg = %Bar.Abc.Msg{a: 42}\n```","title":"Prepend namespaces - Protox","ref":"readme.html#prepend-namespaces"},{"type":"extras","doc":"One or more include paths (directories in which to search for imports) can be specified using the `:paths` option:\n\n```elixir\ndefmodule Baz do\n  use Protox,\n    files: [\n      \"./defs1/prefix/foo.proto\",\n      \"./defs1/prefix/bar.proto\",\n      \"./defs2/prefix/baz/baz.proto\"\n    ],\n    paths: [\n      \"./defs1\",\n      \"./defs2\"\n    ]\nend\n```\n\n> [!NOTE]\n> It corresponds to the `-I` option of protoc.","title":"Specify include path - Protox","ref":"readme.html#specify-include-path"},{"type":"extras","doc":"It's possible to generate Elixir source code files with the mix task `protox.generate`:\n\n```shell\nprotox.generate --output-path=/path/to/messages.ex protos/foo.proto protos/bar.proto\n```\n\nThe files will be usable in any project as long as Protox is declared in the dependencies as functions from its runtime are used.\n\n> [!NOTE]\n> protoc is not needed to compile the generated files.","title":"Files generation - Protox","ref":"readme.html#files-generation"},{"type":"extras","doc":"* `--output-path`\n\n  The path to the file to be generated or to the destination folder when generating multiple files.\n\n* `--include-path`\n\n  Specifies the [include path](#specify-include-path). If multiple include paths are needed, add more `--include-path` options.\n\n* `--multiple-files`\n\n  Generates one file per Elixir module. It's useful for definitions with a lot of messages as the compilation will be parallelized.\n  When generating multiple files, the `--output-path` option must point to a directory.\n\n* `--namespace`\n\n  [Prepends a namespace](#prepend-namespaces) to all generated modules.","title":"Options - Protox","ref":"readme.html#options"},{"type":"extras","doc":"[Unknown fields](https://developers.google.com/protocol-buffers/docs/proto3#unknowns) are fields that are present on the wire but which do not correspond to an entry in the protobuf definition. Typically, it occurs when the sender has a newer version of the protobuf definition. It enables backwards compatibility as the receiver with an old version of the protobuf definition will still be able to decode old fields.\n\nWhen unknown fields are encountered at decoding time, they are kept in the decoded message. It's possible to access them with the  `unknown_fields/1` function defined with the message.\n\n```elixir\niex> msg = Msg.decode!(<<8, 42, 42, 4, 121, 97, 121, 101, 136, 241, 4, 83>>)\n%Msg{a: 42, b: \"\", z: -42, __uf__: [{5, 2, <<121, 97, 121, 101>>}]}\n\niex> Msg.unknown_fields(msg)\n[{5, 2, <<121, 97, 121, 101>>}]\n```\n\nYou must use `unknown_fields/1` as the name of the field (e.g. `__uf__` in the above example) is generated at compile-time to avoid collision with the actual fields of the Protobuf message. This function returns a list of tuples `{tag, wire_type, bytes}`. For more information, please see the [protobuf encoding guide](https://developers.google.com/protocol-buffers/docs/encoding).\n\n> [!NOTE]\n> Unknown fields are retained when re-encoding the message.","title":"Unknown fields - Protox","ref":"readme.html#unknown-fields"},{"type":"extras","doc":"* The [Any](https://developers.google.com/protocol-buffers/docs/reference/google.protobuf#any) well-known type is partially supported: you can manually unpack the embedded message after decoding and conversely pack it before encoding;\n* Groups ([deprecated in protobuf](https://protobuf.dev/programming-guides/proto2/#groups));\n* All [options](https://developers.google.com/protocol-buffers/docs/proto3#options) other than `packed` and `default` are ignored as they concern other languages implementation details.","title":"Unsupported features - Protox","ref":"readme.html#unsupported-features"},{"type":"extras","doc":"*  (__Protobuf 2__) __Required fields__ Protox enforces the presence of required fields; an error is raised when encoding a message with missing required field:\n    ```elixir\n    defmodule Bar do\n      use Protox, schema: \"\"\"\n        syntax = \"proto2\";\n\n        message Required {\n          required int32 a = 1;\n        }\n      \"\"\"\n    end\n\n    iex> Protox.encode!(%Required{})\n    ** (Protox.RequiredFieldsError) Some required fields are not set: [:a]\n    ```\n\n* __Enum aliases__ When decoding, the last encountered constant is used. For instance, in the following example, `:BAR` is always used if the value `1` is read on the wire:\n    ```protobuf\n    enum E {\n      option allow_alias = true;\n      FOO = 0;\n      BAZ = 1;\n      BAR = 1;\n    }\n    ```\n\n* (__Protobuf 2__) __Unset optional fields__ are assigned `nil`. You can use the generated `default/1` function to get the default value of a field:\n    ```elixir\n    defmodule Bar do\n      use Protox,\n      schema: \"\"\"\n        syntax = \"proto2\";\n\n        message Foo {\n          optional int32 a = 1 [default = 42];\n        }\n      \"\"\"\n    end\n\n    iex> %Foo{}.a\n    nil\n\n    iex> Foo.default(:a)\n    {:ok, 42}\n    ```\n\n* (__Protobuf 3__) __Unset fields__ are assigned to their [default values](https://developers.google.com/protocol-buffers/docs/proto3#default). However, if you use the `optional` keyword (available in protoc >= 3.15), then unset fields are assigned `nil`:\n    ```elixir\n    defmodule Bar do\n      use Protox,\n      schema: \"\"\"\n        syntax = \"proto3\";\n\n        message Foo {\n          int32 a = 1;\n          optional int32 b = 2;\n        }\n      \"\"\"\n    end\n\n    iex> %Foo{}.a\n    0\n\n    iex> Foo.default(:a)\n    {:ok, 0}\n\n    iex> %Foo{}.b\n    nil\n\n    iex> Foo.default(:b)\n    {:error, :no_default_value}\n    ```\n\n* __Messages and enums names__ are converted using the [`Macro.camelize/1`](https://hexdocs.pm/elixir/Macro.html#camelize/1) function.\n  Thus, in the following example, `non_camel_message` becomes `NonCamelMessage`, but the field `non_camel_field` is left unchanged:\n    ```elixir\n    defmodule Bar do\n      use Protox,\n      schema: \"\"\"\n        syntax = \"proto3\";\n\n        message non_camel_message {\n        }\n\n        message CamelMessage {\n          int32 non_camel_field = 1;\n        }\n      \"\"\"\n    end\n\n    iex> msg = %NonCamelMessage{}\n    %NonCamelMessage{__uf__: []}\n\n    iex> msg = %CamelMessage{}\n    %CamelMessage{__uf__: [], non_camel_field: 0}\n    ```","title":"Implementation choices - Protox","ref":"readme.html#implementation-choices"},{"type":"extras","doc":"- The detailed reference of the generated code is available in [documentation/reference.md](documentation/reference.md).\n- Please see [documentation/types_mapping.md](documentation/types_mapping.md) to see how protobuf types are mapped to Elixir types.","title":"Generated code reference and types mapping - Protox","ref":"readme.html#generated-code-reference-and-types-mapping"},{"type":"extras","doc":"The Protox library has been thoroughly tested using the conformance checker [provided by Google](https://github.com/protocolbuffers/protobuf/tree/master/conformance).\n\nTo launch these conformance tests, use the `protox.conformance` mix task:\n  ```\n  $ mix protox.conformance\n  WARNING: All log messages before absl::InitializeLog() is called are written to STDERR\n  I0000 00:00:1738246114.224098 3490144 conformance_test_runner.cc:394] ./protox_conformance\n  CONFORMANCE TEST BEGIN ====================================\n\n  CONFORMANCE SUITE PASSED: 1368 successes, 1307 skipped, 0 expected failures, 0 unexpected failures.\n\n  WARNING: All log messages before absl::InitializeLog() is called are written to STDERR\n  I0000 00:00:1738246115.065491 3495574 conformance_test_runner.cc:394] ./protox_conformance\n  CONFORMANCE TEST BEGIN ====================================\n\n  CONFORMANCE SUITE PASSED: 0 successes, 414 skipped, 0 expected failures, 0 unexpected failures.\n  ```\n\n>[!NOTE]\n> A report will be generated in the directory `conformance_report`.","title":"Conformance - Protox","ref":"readme.html#conformance"},{"type":"extras","doc":"Please see [benchmark/launch_benchmark.md](benchmark/launch_benchmark.md) for more information on how to launch benchmark.","title":"Benchmark - Protox","ref":"readme.html#benchmark"},{"type":"extras","doc":"Please see [CONTRIBUTING.md](./CONTRIBUTING.md) for more information on how to contribute.","title":"Contributing - Protox","ref":"readme.html#contributing"},{"type":"extras","doc":"# Protox Benchmark\n\nAll the following commands are executed from the root of the project.","title":"Protox Benchmark","ref":"launch_benchmark.html"},{"type":"extras","doc":"- you can specify the task to run (`encode` or `decode`)\n- you have to specify the benchmark tag, which will be used to identify the benchmark run\n- benchmark results are stored in `./benchmark/output/benchee/ - - `\n\n```\nmix protox.benchmark.run [--task  ]  \n```\n\nExample:\n\n```\nmix protox.benchmark.run --task decode my_tag\n```","title":"Launch the benchmark - Protox Benchmark","ref":"launch_benchmark.html#launch-the-benchmark"},{"type":"extras","doc":"```\nmix protox.benchmark.report  \n```\n\nExample:\n\n```\nmix protox.benchmark.report ./benchmark/output/benchee/encode-*\n```","title":"Aggregate the results - Protox Benchmark","ref":"launch_benchmark.html#aggregate-the-results"},{"type":"extras","doc":"# Generated code reference\n\nThis documentation lists the functions generated with each Elixir structure associated to a protobuf [message](documentation/reference.md#messages) or [enum](documentation/reference.md#enums).","title":"Generated code reference","ref":"reference.html"},{"type":"extras","doc":"","title":"Messages - Generated code reference","ref":"reference.html#messages"},{"type":"extras","doc":"```elixir\n@spec encode(struct() :: {:ok, iodata()} | {:error, any()}\nencode(msg)\n```\n\nEncode `msg` into an `iodata` suitable for files or sockets.\nReturns `{:ok, iodata}` when the encoding was successful and `{:error, description}` in case of an encoding error.\n\n\n```elixir\n@spec encode!(struct() :: iodata() | no_return()\nencode!(msg)\n```\n\nThrowing version of `encode/1`.","title":"Encode - Generated code reference","ref":"reference.html#encode"},{"type":"extras","doc":"```elixir\n@spec decode(binary() :: {:ok, struct()} | {:error, any()}\ndecode(data)\n```\n\nDecode binary `data` into an structure with the type of the module on which this function is called.\nReturns `{:ok, msg}` when the decoding was successful and `{:error, description}` in case of an decoding error.\n\n\n```elixir\n@spec decode!(binary() :: struct() | no_return()\ndecode!(data)\n```\n\nThrowing version of `decode/1`.","title":"Decode - Generated code reference","ref":"reference.html#decode"},{"type":"extras","doc":"```elixir\n@spec default(atom() :: {:ok, boolean() | integer() | String.t() | binary() | float()} | {:error, atom()}\ndefault(field_name)\n```\n\nGet the default value of a message field. Note that for Protobuf 3, the default value is mandated by [the Google reference documentation](https://developers.google.com/protocol-buffers/docs/proto3#default).","title":"Default values - Generated code reference","ref":"reference.html#default-values"},{"type":"extras","doc":"```elixir\n@spec clear_unknown_fields(struct() :: struct()\nclear_unknown_fields(msg)\n```\nReturns a copy of `msg` with all unknown fields removed.\n\n```elixir\n@spec unknown_fields(struct() :: [{non_neg_integer(), Protox.Types.tag(), binary()}]\nunknown_fields(msg)\n```\nGet the unknown fields that may have been encountered when decoding data.\nSee [Types](documentation/reference.md#types) section to get a description of `Protox.Types.tag`.\n\n```elixir\n@spec unknown_fields_name() :: atom()\nunknown_fields_name(msg)\n```\nGet the name of the field that stores unknown fields.","title":"Unknown fields - Generated code reference","ref":"reference.html#unknown-fields"},{"type":"extras","doc":"```elixir\n@spec schema() :: Protox.MessageSchema.t()\nschema()\n```\nReturn the underlying definition of a message, which contains information such as:\n- syntax (protobuf 2 or 3)\n- required fields\n- types of fields","title":"Metadata - Generated code reference","ref":"reference.html#metadata"},{"type":"extras","doc":"```elixir\n@spec default() :: atom()\ndefault()\n```\nGet the default value of an enum.\n\n```elixir\n@spec encode(atom() :: integer() | atom()\nencode(enum_entry)\n```\nGet the integer value of an enum entry. If `enum_entry` does not exist in the enum, it is returned as is.\n\n\n```elixir\n@spec decode(integer() :: atom() | integer()\ndecode(value)\n```\nGet the enum entry of an integer value. If `value` does not correpond to any entry in the enum, it is returned as is.\n\n```elixir\n@spec constants() :: [{integer(), atom()}]\nconstants()\n```\nGet the list of all the constants of the enum that correponds to the module on which this function has been called.","title":"Enums - Generated code reference","ref":"reference.html#enums"},{"type":"extras","doc":"# Types mapping\n\nThe following table shows how Protobuf types are mapped to Elixir's ones.\n\nProtobuf   | Elixir\n---------- | -------------\n`int32`    | `integer()`\n`int64`    | `integer()`\n`uint32`   | `integer()`\n`uint64`   | `integer()`\n`sint32`   | `integer()`\n`sint64`   | `integer()`\n`fixed32`  | `integer()`\n`fixed64`  | `integer()`\n`sfixed32` | `integer()`\n`sfixed64` | `integer()`\n`float`    | `float() \\| :infinity \\| :'-infinity' \\| :nan`\n`double`   | `float() \\| :infinity \\| :'-infinity' \\| :nan`\n`bool`     | `boolean()`\n`string`   | `String.t()`\n`bytes`    | `binary()`\n`repeated` | `list(value_type)` where `value_type` is the type of the repeated field\n`map`      | `map()`\n`oneof`    | `{atom(), value_type}` where `atom()` is the type of the set field and where `value_type` is the type of the set field\n`enum`     | `atom() \\| integer()`\n`message`  | `struct()`","title":"Types mapping","ref":"types_mapping.html"},{"type":"extras","doc":"# Migration guide (v1 to v2)\n\nThis guide explains how to migrate from version 1 to version 2 of Protox.\n\n> [!NOTE]\n> You'll find the rationales behind the changes in the [changelog](../CHANGELOG.md).","title":"Migration guide (v1 to v2)","ref":"v1_to_v2_migration.html"},{"type":"extras","doc":"Protox now returns the size of the encoded messages along with the encoded data. If you don't need the size, you can simply ignore it:\n\n```elixir\niex> msg = %Foo{a: 3, b: %{1 => %Baz{}}}\n{:ok, iodata, _iodata_size} = Protox.encode(msg)\n```","title":"Encoding - Migration guide (v1 to v2)","ref":"v1_to_v2_migration.html#encoding"},{"type":"extras","doc":"It's no longer possible to encode or decode JSON data directly using Protox. If it's necessary, you can stick to version 1.7 or switch to [`protobuf`](https://hex.pm/packages/protobuf).\n\n## `Protox` macro options\n\nThe `:path` option is removed in favor of the already existing `:paths` option, thus one just has to provide a list containing a single path.\n\nAlso, the `:keep_unknown_fields` option is no longer available. Thus, unknown fields are always kept. If you don't need them, you can simply ignore them.","title":"JSON support - Migration guide (v1 to v2)","ref":"v1_to_v2_migration.html#json-support"},{"type":"extras","doc":"The following functions generated for messages are replaced by the function `schema/0`:\n\n- `defs/0`\n- `field_def/1`\n- `file_options/0`\n- `required_fields/0`\n- `syntax/0`\n\n`schema/0` returns a `Protox.MessageSchema` struct which contains information about the message's fields, syntax, and file options.","title":"Generated code - Migration guide (v1 to v2)","ref":"v1_to_v2_migration.html#generated-code"},{"type":"extras","doc":"```elixir\niex> defmodule MyModule do\n  use Protox, schema: \"\"\"\n  syntax = \"proto2\";\n\n  message Foo {\n    required int32 a = 1;\n    map  b = 2;\n  }\n  \"\"\"\nend\n\niex> Foo.schema().syntax\n:proto2\n\niex> Foo.schema().fields[:a]\n%Protox.Field{\n  tag: 1,\n  label: :required,\n  name: :a,\n  kind: %Protox.Scalar{default_value: 0},\n  type: :int32\n}\n\niex> Foo.schema().file_options\nnil\n```","title":"Example - Migration guide (v1 to v2)","ref":"v1_to_v2_migration.html#example"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.3"}}